# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_bkmr_global_optspecs
	string join \n c/config= d/debug openai generate-config h/help V/version
end

function __fish_bkmr_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_bkmr_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_bkmr_using_subcommand
	set -l cmd (__fish_bkmr_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c bkmr -n "__fish_bkmr_needs_command" -s c -l config -d 'Sets a custom config file' -r -F
complete -c bkmr -n "__fish_bkmr_needs_command" -s d -l debug -d 'Turn debugging information on'
complete -c bkmr -n "__fish_bkmr_needs_command" -l openai -d 'use OpenAI API to embed bookmarks'
complete -c bkmr -n "__fish_bkmr_needs_command" -l generate-config -d 'bkmr --generate-config > ~/.config/bkmr/config.toml'
complete -c bkmr -n "__fish_bkmr_needs_command" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_needs_command" -s V -l version -d 'Print version'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "search" -d 'Searches Bookmarks'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "sem-search" -d 'Semantic Search with OpenAI'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "open" -d 'Open/launch bookmarks'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "add" -d 'Add a bookmark'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "delete" -d 'Delete bookmarks'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "update" -d 'Update bookmarks'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "edit" -d 'Edit bookmarks'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "show" -d 'Show Bookmarks (list of ids, separated by comma, no blanks)'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "surprise" -d 'Opens n random URLs'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "tags" -d 'Tag for which related tags should be shown. No input: all tags are printed'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "create-db" -d 'Initialize bookmark database'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "set-embeddable" -d 'Set whether a bookmark can be embedded (used for semantic search)'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "backfill" -d 'Backfill embeddings for bookmarks, which have been added without embeddings. E.g. when OpenAI API was not available'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "load-json" -d 'Load bookmarks from JSON array file'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "load-texts" -d 'Load texts for semantic similarity search as bookmarks. The actual content of the file is not stored in the database, only the embeddings'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "import-files" -d 'Import files from directories, parsing frontmatter metadata'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "info" -d 'Show program information and configuration details'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "completion" -d 'Generate shell completions for bash, zsh, or fish'
complete -c bkmr -n "__fish_bkmr_needs_command" -a "xxx"
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s e -l exact -d 'match exact, comma separated list' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l exact-prefix -d 'tags to prefix the exact option' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s t -l tags -d 'match all, comma separated list' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l tags-prefix -d 'tags to prefix the tags option' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s T -l Tags -d 'not match all, comma separated list' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l Tags-prefix -d 'tags to prefix the Tags option' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s n -l ntags -d 'match any, comma separated list' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l ntags-prefix -d 'tags to prefix the ntags option' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s N -l Ntags -d 'not match any, comma separated list' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l Ntags-prefix -d 'tags to prefix the Ntags option' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l fzf-style -d 'fuzzy finder style: classic or enhanced' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s l -l limit -d 'limit number of results' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s o -l descending -d 'order by age, descending'
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s O -l ascending -d 'order by age, ascending'
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l np -d 'no prompt'
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l fzf -d 'use fuzzy finder: [CTRL-O: copy to clipboard (shell scripts: copy \'bkmr open --no-edit <id> --\' command), CTRL-E: edit, CTRL-D: delete, CTRL-A: clone, CTRL-P: show details, ENTER: open]'
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l json -d 'non-interactive mode, output as json'
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l interpolate -d 'process template interpolation in search results display (not needed for FZF mode or bookmark actions - they interpolate automatically)'
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -l shell-stubs -d 'output shell function stubs for shell script bookmarks (automatically filters for _shell_ type)'
complete -c bkmr -n "__fish_bkmr_using_subcommand search" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand sem-search" -s l -l limit -d 'limit number of results' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand sem-search" -l np -d 'no prompt'
complete -c bkmr -n "__fish_bkmr_using_subcommand sem-search" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand open" -l no-edit -d 'skip interactive editing for shell scripts'
complete -c bkmr -n "__fish_bkmr_using_subcommand open" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -l title -d 'title' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -s d -l description -d 'title' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -s t -l type -d 'bookmark type (uri, snip, text, shell, md, env)' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -s c -l clone -d 'clone an existing bookmark by ID' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -l no-web -d 'do not fetch URL data'
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -s e -l edit -d 'edit the bookmark while adding'
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -l stdin -d 'read content from stdin into url field'
complete -c bkmr -n "__fish_bkmr_using_subcommand add" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand delete" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand update" -s t -l tags -d 'add tags to taglist' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand update" -s n -l ntags -d 'remove tags from taglist' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand update" -s f -l force -d 'overwrite taglist with tags'
complete -c bkmr -n "__fish_bkmr_using_subcommand update" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand edit" -l force-db -d 'force edit database content instead of source file for file-imported bookmarks'
complete -c bkmr -n "__fish_bkmr_using_subcommand edit" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand show" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand surprise" -s n -d 'number of URLs to open' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand surprise" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand tags" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand create-db" -l pre-fill -d 'Pre-fill the database with demo entries'
complete -c bkmr -n "__fish_bkmr_using_subcommand create-db" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand set-embeddable" -l enable -d 'Enable embedding for this bookmark'
complete -c bkmr -n "__fish_bkmr_using_subcommand set-embeddable" -l disable -d 'Disable embedding for this bookmark'
complete -c bkmr -n "__fish_bkmr_using_subcommand set-embeddable" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand backfill" -s d -l dry-run -d 'only show what would be done'
complete -c bkmr -n "__fish_bkmr_using_subcommand backfill" -s f -l force -d 'force recompute of all embeddings (except _imported_)'
complete -c bkmr -n "__fish_bkmr_using_subcommand backfill" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand load-json" -s d -l dry-run -d 'only show what would be done'
complete -c bkmr -n "__fish_bkmr_using_subcommand load-json" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand load-texts" -s d -l dry-run -d 'only show what would be done'
complete -c bkmr -n "__fish_bkmr_using_subcommand load-texts" -s f -l force -d 'force update embeddings even if content has not changed'
complete -c bkmr -n "__fish_bkmr_using_subcommand load-texts" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand import-files" -l base-path -d 'Base path variable name from config (e.g., SCRIPTS_HOME). Paths must be relative to the base path location.' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand import-files" -s u -l update -d 'Update existing bookmarks when content differs'
complete -c bkmr -n "__fish_bkmr_using_subcommand import-files" -l delete-missing -d 'Delete bookmarks whose source files no longer exist'
complete -c bkmr -n "__fish_bkmr_using_subcommand import-files" -s d -l dry-run -d 'Show what would be done without making changes'
complete -c bkmr -n "__fish_bkmr_using_subcommand import-files" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand info" -s s -l schema -d 'Show database schema'
complete -c bkmr -n "__fish_bkmr_using_subcommand info" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand completion" -s h -l help -d 'Print help'
complete -c bkmr -n "__fish_bkmr_using_subcommand xxx" -s t -l tags -d 'add tags to taglist' -r
complete -c bkmr -n "__fish_bkmr_using_subcommand xxx" -s h -l help -d 'Print help'
